#!/usr/bin/perl

print<<EOF;
#################################################################################
#Copyright 2012 by the contributors						#
#										#
#This program is free software; you can redistribute it and/or modify		#
#it under the terms of the GNU General Public License as published by		#
#the Free Software Foundation; either version 2 of the License, or		#
#(at your option) any later version.						#
#										#
#This program is distributed in the hope that it will be useful,		#
#but WITHOUT ANY WARRANTY; without even the implied warranty of			#
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the			#
#GNU General Public License for more details.					#
#										#
#You should have received a copy of the GNU General Public License		#
#along with this program; if not, write to the Free Software			#
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA	#
#										#
#Wherever third party code has been used, credit has been given in the code's	#
#comments.									#
#										#
#Author: Orkan Yoksulabakan, orki-x\@web.de					#
#Version: 0.0.3,  09 september 2012						#
#################################################################################
EOF

use Algorithm::LUHN qw/check_digit is_valid/;
use Math::BigInt;

$version = "$0 - version 0.0.3";

if ($#ARGV != 1) 
{
	print_main_menu();
}
else
{
	#For Quick execution of PAN computing, you can enter the right parameters at time of execution of the program. 
	#This causes the program to start computing immediately instead of calling the main menu. For valid Input, please check valid strings (CC Types) in function validate_input()
	$cc_t=$ARGV[0];
	$outf=$ARGV[1];

	validate_input($cc_t, $outf);
}

sub check_pan{

	print "Please enter PAN: ";
	my $pan = <STDIN>;
	chop ($pan);
	if (is_valid($pan))
	{
		print "\n\nOh oooh! >>>$pan<<< is a valid PAN! );\n\n";
	}
	else
	{
		print "\n\nYou are lucky, $pan is NOT a valid PAN ;)\n\n";
	}
	print_main_menu;
}

sub validate_input{

my $type = $_[0];
my $out = $_[1];
my $cc_start;
my $cc_end;

if ( $type eq "visa13" ) 
{
	$cc_start = 4000000000000;
 	$cc_end   = 4999999999999;
	compute($type, $out, $cc_start, $cc_end);
}
elsif ( $type eq "visa16")
{
	$cc_start = Math::BigInt->new('4000000000000000');
        $cc_end   = Math::BigInt->new('4999999999999999');
	compute($type, $out, $cc_start, $cc_end);
}
elsif ( $type eq "visa19")
{
	$cc_start = Math::BigInt->new('4000000000000000000');
        $cc_end   = Math::BigInt->new('4999999999999999999');
	compute($type, $out, $cc_start, $cc_end);
}
elsif ( $type eq "amex1")
{
	$cc_start = Math::BigInt->new('340000000000000');
	$cc_end   = Math::BigInt->new('349999999999999');
	compute($type, $out, $cc_start, $cc_end);
}
elsif ( $type eq "amex2")
{
	$cc_start = Math::BigInt->new('370000000000000');
	$cc_end   = Math::BigInt->new('379999999999999');
	compute($type, $out, $cc_start, $cc_end);
}
elsif ( $type eq "mc")
{
	$cc_start = Math::BigInt->new('5100000000000000');
	$cc_end   = Math::BigInt->new('5599999999999999');
	compute($type, $out, $cc_start, $cc_end);
}
elsif ( $type eq "q")
{
	exit;
}
else 
{
	print "\n\n\n!!!!!!!!DON't TREAT ME AS A FOOL! WRONG INPUT, TRY AGAIN RIGHT NOW!!!!!!!!\n\n\n";	
	print_menu();
}
}
sub compute {
	
	my $cc_type = $_[0];  #parameter value by function validate_input()
	my $outfile = $_[1];	#parameter value by function validate_input()
	my $cc_num_start = $_[2]; #parameter value by function validate_input()
	my $cc_num_end = $_[3];    #parameter value by function validate_input()
	my $init_seperator="#";      #Field seperator
	my $seperator=$init_seperator; #Initialisation of field seperator for the first turn. U can use any character, but no numbers ;)
	my $number_fields=10;  #Number of fields with PAN in output file. Change it for scaling if needed.
	my $i=1; #counter of fields to add a newline if value of $number_fields is reached (see line 136)
	$outfile="out_$cc_type.txt" if (!$outfile);
	my $path="";	
	#$path=`pwd`; #You can use this if you are not under Microsoft operating system to print the whole path of outputfile to stdout
	#chop ($path); #You can use this if you are not under Microsoft operating system to print the whole path of outputfile to stdout
	print "Creating $path$outfile ...\n";
	open (OUT,">$outfile") or die ("Can't open $outfile , error: $!\n");
	print "This may take a while, don't have a seat until you've brought me a coffee ;)\n";
	

	for($cc=$cc_num_start; $cc<=$cc_num_end; $cc++)
	{
	
		#$csum = check_digit($cc);

		if (is_valid("$cc"))
		{  
		
			$seperator="\n" if ($i == $number_fields);		
			push(@buffer,"$cc$seperator"); #We have one buffer per line. If $number_fields is reached, the new data will be written to outputfile
		
			if ($i == $number_fields)			#max number of fields per line
			{
				print OUT @buffer;
    				@buffer=();
				$i = 0; 		#must be reseted to 0 because of counting up for next turn
				$seperator=$init_seperator;  #reset to default
			}
			$i++;
		}

#OPTIONAL STDOUT. If written in output file, the file will be much bigger, of course we just want the valid ones to be written ;)
#		elsif (!(is_valid("$cc")))
#		{
#			print "$Algorithm::LUHN::ERROR"."\n";
#		}	
#	
	}

print "Done. Please find computed numbers in $outfile \n\n";
close (OUT);
print_menu();
}

sub print_menu {

 	print "\n( Quick usage for computing:#>perl $0 <CC Type> <Output filename> )\n";
	print "CC Type must be one of the following (case sensitive): \n\n";
	print "For 13 digit Visa:            visa13\n";
	print "For 16 digit Visa:            visa16\n";
	print "For 19 digit Visa:            visa19\n";
	print "For Amex starting with 34:    amex1\n";
	print "For Amex starting with 37:    amex2\n";
	print "For MC:                       mc\n\n";
	print "To return to main menu, please typ m\n\n";
	print "To quit, type q or use strg+C\n\n";
	print "Please enter your CC Type: ";
	$cc_t = <STDIN>;
	chop($cc_t);
	exit if ($cc_t eq "q");
	print_main_menu () if ($cc_t eq "m");
	print "Please enter Output filename (or leave empty): ";
	$outf = <STDIN>;
	chop($outf);
	validate_input($cc_t, $outf);
}

sub print_main_menu {
	print "\n\n######CREATE AND VALIDATE PAN#######\n";
	print "########### MAIN MENU ##############\n";
	print "1 for PAN Validation\n";
	print "2 for PAN computing menu\n";
	print "3 for program version\n";
	print "q to quit\n";
	print "Please enter your choise: ";
	my $option = <STDIN>;
	chop($option);
	
	check_pan if ($option == 1);
	print_menu  if ($option == 2);
	print "\n$version\n" if ($option == 3);
	exit        if ($option eq "q");
	print_main_menu();
}
exit;

